#include "CoordinateSystem3DZoomer.h"

//----------------------------------------------------------------------------
// construction/destruction:

CoordinateSystem3DZoomer::CoordinateSystem3DZoomer()
{
 // init member variables:
 theCoordinateSystem3D  = NULL;
 supressZoomability     = false;
 zoomerSize             = 16;
 verticalMouseWheelMode = noResponseToVerticalMouseWheel;

 zoomFactorPerClickX = 1.25;
 zoomFactorPerClickY = 1.25;
 shiftPerClickX      = 0.1;
 shiftPerClickY      = 0.1;

 oldMouseDistanceX   = 0;
 oldMouseDistanceY   = 0;

 // create the horizontal zoom-buttons and ScrollBar:
 zoomInButtonX = new SymbolButton();
 zoomInButtonX->setTooltip(String(T("zoom in horizontally")));
 zoomInButtonX->setSymbolIndex(SymbolButton::PLUS);
 zoomInButtonX->setClickingTogglesState(false);
 zoomInButtonX->addButtonListener(this);
 addAndMakeVisible(zoomInButtonX);

 zoomToAllButtonX = new ToggleButton(String("h"));
 zoomToAllButtonX->setTooltip(String(T("zoom maximally out horizontally")));
 zoomToAllButtonX->setClickingTogglesState(false);
 //zoomToAllButtonX->setConnectedEdges(Button::ConnectedOnLeft);
 //zoomToAllButtonX->setConnectedEdges(Button::ConnectedOnLeft|
 //                                    Button::ConnectedOnRight|
 //                                    Button::ConnectedOnTop|
 //                                    Button::ConnectedOnBottom);
 zoomToAllButtonX->addButtonListener(this);
 addAndMakeVisible(zoomToAllButtonX);

 zoomOutButtonX = new SymbolButton();
 zoomOutButtonX->setTooltip(String(T("zoom out horizontally")));
 zoomOutButtonX->setSymbolIndex(SymbolButton::MINUS);
 zoomOutButtonX->setClickingTogglesState(false);
 zoomOutButtonX->addButtonListener(this);
 addAndMakeVisible(zoomOutButtonX);

 scrollBarX = new ScrollBar(false, true);
 scrollBarX->setRangeLimits(-1.0, +1.0);
 scrollBarX->addListener(this);
 addAndMakeVisible(scrollBarX);

 // create the vertical zoom-buttons and ScrollBar:
 zoomInButtonY = new SymbolButton();
 zoomInButtonY->setTooltip(String(T("zoom in vertically")));
 zoomInButtonY->setSymbolIndex(SymbolButton::PLUS);
 zoomInButtonY->setClickingTogglesState(false);
 zoomInButtonY->addButtonListener(this);
 addAndMakeVisible(zoomInButtonY);

 zoomToAllButtonY = new ToggleButton(String("v"));
 zoomToAllButtonY->setTooltip(String(T("zoom maximally out vertically")));
 zoomToAllButtonY->setClickingTogglesState(false);
 zoomToAllButtonY->addButtonListener(this);
 addAndMakeVisible(zoomToAllButtonY);

 zoomOutButtonY = new SymbolButton();
 zoomOutButtonY->setTooltip(String(T("zoom out vertically")));
 zoomOutButtonY->setSymbolIndex(SymbolButton::MINUS);
 zoomOutButtonY->setClickingTogglesState(false);
 zoomOutButtonY->addButtonListener(this);
 addAndMakeVisible(zoomOutButtonY);

 scrollBarY = new ScrollBar(true, true);
 scrollBarY->setRangeLimits(-1.0, +1.0);
 scrollBarY->addListener(this);
 addAndMakeVisible(scrollBarY);

 // create the zoom-to-all button for X- and Y:
 zoomToAllButtonXY = new ToggleButton(String("a"));
 zoomToAllButtonXY->setTooltip(String(T("zoom maximally out both axes")));
 zoomToAllButtonXY->setClickingTogglesState(false);
 zoomToAllButtonXY->addButtonListener(this);
 addAndMakeVisible(zoomToAllButtonXY);
}

CoordinateSystem3DZoomer::~CoordinateSystem3DZoomer()
{
 deleteAllChildren();
}

//----------------------------------------------------------------------------
// parameter-settings:

void CoordinateSystem3DZoomer::setCoordinateSystem3D(CoordinateSystem3D
                                                   *newSystemToBeShown)
{
 // remove and delete the old CoordinateSystem3D (if any present):
 if( theCoordinateSystem3D != NULL )
 {
  delete theCoordinateSystem3D;
  theCoordinateSystem3D = NULL;
 }

 // assign the pointer to the new CoordinateSystem3D:
 theCoordinateSystem3D = newSystemToBeShown;

 // align the widgets::
 alignWidgetsToCoordinateSystem3D();

 // update the scrollbar ranges:
 scrollBarX->setRangeLimits(-1.0, +1.0);
 scrollBarX->setCurrentRange(-1.0, 2.0);
 scrollBarY->setRangeLimits(-1.0, +1.0);
 scrollBarY->setCurrentRange(-1.0, 2.0);
}

void CoordinateSystem3DZoomer::invalidateCoordinateSystem3DPointer()
{
 theCoordinateSystem3D = NULL;
}

void CoordinateSystem3DZoomer::setVerticalMouseWheelMode(int newMode)
{
 verticalMouseWheelMode = newMode;
}




//----------------------------------------------------------------------------
// callbacks:

void CoordinateSystem3DZoomer::buttonClicked(Button *button)
{
 // check, if we have a valid pointer to a CoordinateSystem3D:
 if( theCoordinateSystem3D == NULL ) 
  return;

 if( button == zoomInButtonX )
  zoomX(zoomFactorPerClickX);  
 else if( button == zoomOutButtonX )
  zoomX(1.f/zoomFactorPerClickX);  
 else if( button == zoomToAllButtonX )
 {
  zoomToAllX();
 }
 else if( button == zoomInButtonY )
  zoomY(zoomFactorPerClickY);  
 else if( button == zoomOutButtonY )
  zoomY(1.f/zoomFactorPerClickY);  
 else if( button == zoomToAllButtonY )
 {
  zoomToAllY();
 }
 else if( button == zoomToAllButtonXY )
 {
  zoomToAllXY();
 }

 // update the scrollbars according to the new zoom-levels:
 updateScrollbars();
}

void CoordinateSystem3DZoomer::scrollBarMoved(ScrollBar *scrollBarThatHasMoved, 
                                            const double newRangeStart)
{
 // check, if we have a valid pointer to a CoordinateSystem3D:
 if( theCoordinateSystem3D == NULL ) 
  return;

 /*
 if( scrollBarThatHasMoved == scrollBarX )
 {
  currentMinX = scrollBarX->getCurrentRangeStart();
  currentMaxX = currentMinX + scrollBarX->getCurrentRangeSize();
 }
 else if( scrollBarThatHasMoved == scrollBarY )
 {
  double tmp  =   scrollBarY->getCurrentRangeStart() 
                + scrollBarY->getCurrentRangeSize();
  tmp         = scrollBarY->getMaximumRangeLimit() - tmp;
  currentMinY = scrollBarY->getMinimumRangeLimit() + tmp;
  currentMaxY = currentMinY + scrollBarY->getCurrentRangeSize();

 }

 // restrict the maximum zoom-out-factor to the range-limits:
 restrictCurrentRangeToMaximumRange();

 // update the coordinate-system according to the new range:
 theCoordinateSystem3D->setRange(currentMinX, currentMaxX, 
                               currentMinY, currentMaxY);

 theCoordinateSystem3D->repaint(); // necesarry??
 */
}

void CoordinateSystem3DZoomer::mouseWheelMove(const MouseEvent& e,  
                                            float wheelIncrementX, 
                                            float wheelIncrementY)
{
 if( verticalMouseWheelMode == noResponseToVerticalMouseWheel )
  return; // nothing to do
 else if( verticalMouseWheelMode == forwardToCoordinateSystem3D )
 {
  if( theCoordinateSystem3D != NULL )
   theCoordinateSystem3D->mouseWheelMove(e, wheelIncrementX, wheelIncrementY);
  return;
 }

 if( wheelIncrementY > 0.f )
 {
  switch( verticalMouseWheelMode )
  {
  case horizontalScrollViaVerticalMouseWheel:
   shiftX(shiftPerClickX);
   break;
  case verticalScrollViaVerticalMouseWheel:
   shiftY(shiftPerClickY);
   break;
  case horizontalZoomViaVerticalMouseWheel:
   buttonClicked(zoomInButtonX); 
   break;
  case verticalZoomViaVerticalMouseWheel:
   buttonClicked(zoomInButtonY); 
   break;
  case zoomViaVerticalMouseWheel:
   {
    buttonClicked(zoomInButtonX);
    buttonClicked(zoomInButtonY);
   } break;
  } // end of switch( verticalMouseWheelMode )
 } // end of  if( wheelIncrementY > 0.f )
 else if( wheelIncrementY < 0.f )
 {
  switch( verticalMouseWheelMode )
  {
  case horizontalScrollViaVerticalMouseWheel:
   shiftX(-shiftPerClickX);
   break;
  case verticalScrollViaVerticalMouseWheel:
   shiftY(-shiftPerClickY);
   break;
  case horizontalZoomViaVerticalMouseWheel:
   buttonClicked(zoomOutButtonX); 
   break;
  case verticalZoomViaVerticalMouseWheel:
   buttonClicked(zoomOutButtonY); 
   break;
  case zoomViaVerticalMouseWheel:
   {
    buttonClicked(zoomOutButtonX);
    buttonClicked(zoomOutButtonY);
   } break;
  } // end of switch( verticalMouseWheelMode )
 } // end of else if( wheelIncrementY < 0.f )
}

// forwarding of mouse-events to the CoordinateSystem3D which are not handled
// by the zoomer:
void CoordinateSystem3DZoomer::mouseMove(const MouseEvent& e) 
{ 
 if (theCoordinateSystem3D != NULL) 
  theCoordinateSystem3D->mouseMove(e); 
}
void CoordinateSystem3DZoomer::mouseEnter(const MouseEvent& e) 
{ 
 if (theCoordinateSystem3D != NULL) 
  theCoordinateSystem3D->mouseEnter(e); 
}
void CoordinateSystem3DZoomer::mouseExit(const MouseEvent& e) 
{ 
 if (theCoordinateSystem3D != NULL) 
  theCoordinateSystem3D->mouseExit(e); 
}
void CoordinateSystem3DZoomer::mouseDown(const MouseEvent& e) 
{
 if (theCoordinateSystem3D != NULL) 
 {
  if( e.getNumberOfClicks() == 2 )
  {
   zoomToAllXY();
   theCoordinateSystem3D->useStandardProjection(CoordinateSystem3D::ISOMETRIC);
  }
 }
}
void CoordinateSystem3DZoomer::mouseDrag(const MouseEvent& e) 
{ 
 if (theCoordinateSystem3D != NULL) 
 {
  int x = e.getDistanceFromDragStartX() - oldMouseDistanceX;
  int y = e.getDistanceFromDragStartY() - oldMouseDistanceY;

  theCoordinateSystem3D->incrementRotationAngle(0.2*x, 
   CoordinateSystem3D::Y_AXIS);
  theCoordinateSystem3D->incrementRotationAngle(0.2*y, 
   CoordinateSystem3D::X_AXIS);

  oldMouseDistanceX = e.getDistanceFromDragStartX();
  oldMouseDistanceY = e.getDistanceFromDragStartY();
 }
}

void CoordinateSystem3DZoomer::mouseUp(const MouseEvent& e) 
{ 
 /*
 if (theCoordinateSystem3D != NULL) 
  theCoordinateSystem3D->mouseUp(e); 
  */

 if (theCoordinateSystem3D != NULL) 
 {
  oldMouseDistanceX = 0;
  oldMouseDistanceY = 0;
 }
}

void CoordinateSystem3DZoomer::mouseDoubleClick(const MouseEvent& e) 
{ 
 if (theCoordinateSystem3D != NULL) 
  theCoordinateSystem3D->mouseDoubleClick(e); 
}

//----------------------------------------------------------------------------
// manipulation of the coordinate-system:

void CoordinateSystem3DZoomer::shiftX(double shiftAmount)
{
 if( theCoordinateSystem3D == NULL ) 
  return;

 theCoordinateSystem3D->incrementHorizontalShift(shiftPerClickX);

 // update the x-axis scrollbar according to the new position:
 updateScrollbars();
}

void CoordinateSystem3DZoomer::shiftY(double shiftAmount)
{
 if( theCoordinateSystem3D == NULL ) 
  return;

 theCoordinateSystem3D->incrementVerticalShift(shiftPerClickY);

 // update the y-axis scrollbar according to the new position:
 updateScrollbars();
}

void CoordinateSystem3DZoomer::zoomX(double zoomFactor)
{
 // check, if we have a valid pointer to a CoordinateSystem3D:
 if( theCoordinateSystem3D == NULL ) 
  return;

 /*
 // get the current center-value and the deviations form it to the left and 
 // right:
 double centerX  = 0.5f*(currentMinX+currentMaxX);
 double left     = centerX-currentMinX;    // deviation to the left
 double right    = currentMaxX-centerX;    // deviation to the right

 // shrink/expand the deviations and obtain new minimum and maximum values:
 currentMinX = centerX - left/zoomFactor;
 currentMaxX = centerX + right/zoomFactor;
 */

 theCoordinateSystem3D->scaleHorizontalScaleFactor(zoomFactor);

 // update the x-axis scrollbar according to the new zoom-level:
 updateScrollbars();
}

void CoordinateSystem3DZoomer::zoomY(double zoomFactor)
{
 // check, if we have a valid pointer to a CoordinateSystem3D:
 if( theCoordinateSystem3D == NULL ) 
  return;

 /*
 // get the current center-value and the deviations form it to above and 
 // below:
 double centerY = 0.5f*(currentMinY+currentMaxY);
 double down    = centerY-currentMinY;    // downward deviation
 double up      = currentMaxY-centerY;    // upward deviation

 // shrink/expand the deviations and obtain new minimum and maximum values:
 currentMinY = centerY - down/zoomFactor;
 currentMaxY = centerY + up/zoomFactor;
 */

 theCoordinateSystem3D->scaleVerticalScaleFactor(zoomFactor);

 // update the y-axis scrollbar according to the new zoom-level:
 updateScrollbars();
}

void CoordinateSystem3DZoomer::zoomToAllX()
{
 // check, if we have a valid pointer to a CoordinateSystem3D:
 if( theCoordinateSystem3D == NULL ) 
  return;

 theCoordinateSystem3D->setHorizontalScaleFactor(0.57);
 theCoordinateSystem3D->setHorizontalShift(0.0);

 // update the y-axis scrollbar according to the new zoom-level:
 updateScrollbars();
}

void CoordinateSystem3DZoomer::zoomToAllY()
{
 // check, if we have a valid pointer to a CoordinateSystem3D:
 if( theCoordinateSystem3D == NULL ) 
  return;

 theCoordinateSystem3D->setVerticalScaleFactor(0.57);
 theCoordinateSystem3D->setVerticalShift(0.0);

 // update the y-axis scrollbar according to the new zoom-level:
 updateScrollbars();
}

void CoordinateSystem3DZoomer::zoomToAllXY()
{
 // check, if we have a valid pointer to a CoordinateSystem3D:
 if( theCoordinateSystem3D == NULL ) 
  return;

 theCoordinateSystem3D->setHorizontalScaleFactor(0.57);
 theCoordinateSystem3D->setHorizontalShift(0.0);
 theCoordinateSystem3D->setVerticalScaleFactor(0.57);
 theCoordinateSystem3D->setVerticalShift(0.0);

 // update the y-axis scrollbar according to the new zoom-level:
 updateScrollbars();
}

//----------------------------------------------------------------------------
// setup appearance:

void CoordinateSystem3DZoomer::alignWidgetsToCoordinateSystem3D()
{
 // check, if we have a valid pointer to a CoordinateSystem3D:
 if( theCoordinateSystem3D == NULL ) 
  return;

 setBounds(theCoordinateSystem3D->getX(), 
           theCoordinateSystem3D->getY(), 
           theCoordinateSystem3D->getWidth()+zoomerSize, 
           theCoordinateSystem3D->getHeight()+zoomerSize);

 int x,y,w,h;

 x = theCoordinateSystem3D->getWidth()-zoomerSize;
 y = theCoordinateSystem3D->getHeight();
 zoomInButtonX->setBounds(x, y, zoomerSize, zoomerSize);

 x = theCoordinateSystem3D->getWidth()-2*zoomerSize;
 y = theCoordinateSystem3D->getHeight();
 zoomToAllButtonX->setBounds(x, y, zoomerSize, zoomerSize);

 x = theCoordinateSystem3D->getWidth()-3*zoomerSize;
 y = theCoordinateSystem3D->getHeight();
 zoomOutButtonX->setBounds(x, y, zoomerSize, zoomerSize);

 x = 0;
 y = theCoordinateSystem3D->getHeight();
 w = theCoordinateSystem3D->getWidth()-3*zoomerSize;
 h = zoomerSize;
 scrollBarX->setBounds(x, y, w, h);

 x = theCoordinateSystem3D->getWidth();
 y = theCoordinateSystem3D->getHeight()-zoomerSize;
 zoomInButtonY->setBounds(x, y, zoomerSize, zoomerSize);

 x = theCoordinateSystem3D->getWidth();
 y = theCoordinateSystem3D->getHeight()-2*zoomerSize;
 zoomToAllButtonY->setBounds(x, y, zoomerSize, zoomerSize);

 x = theCoordinateSystem3D->getWidth();
 y = theCoordinateSystem3D->getHeight()-3*zoomerSize;
 zoomOutButtonY->setBounds(x, y, zoomerSize, zoomerSize);

 x = theCoordinateSystem3D->getWidth();
 y = 0;
 w = zoomerSize;
 h = theCoordinateSystem3D->getHeight()-3*zoomerSize;
 scrollBarY->setBounds(x, y, w, h);

 x = theCoordinateSystem3D->getWidth();
 y = theCoordinateSystem3D->getHeight();
 zoomToAllButtonXY->setBounds(x, y, zoomerSize, zoomerSize);
}

void CoordinateSystem3DZoomer::updateScrollbars()
{
 double margin = 0.000001;

 /*
 // update the scrollbar ranges (this requires some fiddling around in order 
 // to ensure that the ScrollBars currentRange never exceeds the rangeLimits):
 scrollBarX->setRangeLimits(-DBL_MAX, DBL_MAX);
 scrollBarX->setCurrentRange(currentMinX+margin, 
                             currentMaxX-currentMinX-2.0*margin);
 scrollBarX->setRangeLimits(rangeMinX, rangeMaxX);

 scrollBarY->setRangeLimits(-DBL_MAX, DBL_MAX);
 double scrollBarMinY = rangeMinY + rangeMaxY - currentMaxY;
 scrollBarY->setCurrentRange(scrollBarMinY+margin, 
                             currentMaxY-currentMinY-2.0*margin);
 scrollBarY->setRangeLimits(rangeMinY, rangeMaxY);
 */
}

/*
void CoordinateSystem3DZoomer::updateRangeLimitsFromCoordinateSystem3D()
{
 rangeMinX = theCoordinateSystem3D->getMinX();
 rangeMaxX = theCoordinateSystem3D->getMaxX();
 rangeMinY = theCoordinateSystem3D->getMinY();
 rangeMaxY = theCoordinateSystem3D->getMaxY();
}
*/

//----------------------------------------------------------------------------
// overrides for inherited functions from the component base-class:

void CoordinateSystem3DZoomer::paint(juce::Graphics &g)
{
 //theCoordinateSystem3D->repaint(); // necesarry??

 // fill the whole GUI with a gradient background:
 //MetallicLookAndFeel::fillRectWithMetallicGradient(g, 0, 0, getWidth(), getHeight());

 // draw an enclosing rectangle:
 //g.setColour(juce::Colours::black);
 //g.drawRect(0, 0, getWidth(), getHeight(), 1);
}
