/**

This a class for a CoordinateSystem3D with zooming and scrolling capabilities. 
It has a pointer to the actual CoordinateSystem3D (or some subclass thereof) 
object for which the zooming and scrolling is required. This pointer needs to
bet set up via the setCoordinateSystem3D()-method - otherwise it will be NULL 
and emptyness will be shown.

*/

#ifndef __JUCE_COORDINATESYSTEM3DZOOMER_JUCEHEADER__
#define __JUCE_COORDINATESYSTEM3DZOOMER_JUCEHEADER__

#include <float.h>

#include "CoordinateSystem3D.h"
#include "SymbolButton.h"

class CoordinateSystem3DZoomer : public Component,
                                 public ButtonListener,
                                 public ScrollBarListener
                              
{

public:

 enum verticalMouseWheelModes
 {
  noResponseToVerticalMouseWheel = 0,
  forwardToCoordinateSystem3D,
  horizontalScrollViaVerticalMouseWheel,
  verticalScrollViaVerticalMouseWheel,
  horizontalZoomViaVerticalMouseWheel,
  verticalZoomViaVerticalMouseWheel,
  zoomViaVerticalMouseWheel
 };


          CoordinateSystem3DZoomer();  ///< Constructor.
 virtual ~CoordinateSystem3DZoomer();  ///< Destructor.

 virtual void setCoordinateSystem3D(CoordinateSystem3D* newSystemToBeShown);
 /**< This function should be used to pass the CoordinateSystem3D-object (or an 
      object of some subclass thereof) which should be shown, zoomed and 
      scrolled. */

 virtual void invalidateCoordinateSystem3DPointer();
 /**< This function should be called when the CoordinateSystem3D, to which we
      hold a pointe here, was deleted for some reason. */

 virtual void setVerticalMouseWheelMode(int newMode);
 /**< Chooses, in which way the zoomer should respond to the vertical 
      mousewheel. For the available modes, see verticalMouseWheelModes. */

 virtual void buttonClicked(Button *button);
 /**< Implements the buttonClicked()-method of the ButtonListener 
      base-class. */

 virtual void scrollBarMoved(ScrollBar *scrollBarThatHasMoved, 
                             const double newRangeStart);
 /**< Implements the scrollBarMoved()-method of the ScrollbarListener 
      base-class. */

 virtual void mouseWheelMove(const MouseEvent& e,  
                             float  wheelIncrementX, float  wheelIncrementY);
 /**< Overrides the mouseWheelMove()-method of the indirect MouseListener 
      base-class to allow zooming by mouswheel. */

 virtual void setZoomerSize(int newZoomerSize) { zoomerSize = newZoomerSize; };

 virtual int getZoomerSize() { return zoomerSize; };
 /**< Returns the size of the zoom- and scroll-widgets. This is the edge-length
      of the buttons, and the thickness of the scrollbars. */

 // additionally, we have to override all the other mouse-callbacks in order 
 // to forward them to the CoordinateSystem3D:
 virtual void mouseMove       (const MouseEvent& e);
 virtual void mouseEnter      (const MouseEvent& e);
 virtual void mouseExit       (const MouseEvent& e);
 virtual void mouseDown       (const MouseEvent& e);
 virtual void mouseDrag       (const MouseEvent& e);
 virtual void mouseUp         (const MouseEvent& e);
 virtual void mouseDoubleClick(const MouseEvent& e);

 virtual void shiftX(double shiftAmount);
 /**< Causes the CoordinateSystem3D to be shifted left or right and updates the 
      scrollBarX. */

 virtual void shiftY(double shiftAmount);
 /**< Causes the CoordinateSystem3D to be shifted up or down and updates the 
      scrollBarY. */

 virtual void zoomX(double zoomFactor);
 /**< Zooms the x-coordinate in or out (according to whether the zoomFactor is
      larger or smaller than 1) and updates the scrollBarX. */

 virtual void zoomY(double zoomFactor);
 /**< Zooms the y-coordinate in or out (according to whether the zoomFactor is
      larger or smaller than 1) and updates the scrollBarY. */

 virtual void zoomToAllX();
 /**< Zooms x-axis out maximally.  */

 virtual void zoomToAllY();
 /**< Zooms y-axis out maximally.  */

 virtual void zoomToAllXY();
 /**< Zooms out maximally to show all.  */

 virtual void alignWidgetsToCoordinateSystem3D();
 /**< Causes the widgets (the zoom-buttons and scrollbars) to align themselves 
      according to the position and size of the CoordinateSystem3D object, which
      is being manipulated */

 virtual void paint(juce::Graphics &g);
 /**< Overrides the paint-method of the component base-class. */

protected:

 // embedded components:
 CoordinateSystem3D* theCoordinateSystem3D;
 SymbolButton*       zoomInButtonX;
 SymbolButton*       zoomOutButtonX;
 //TabBarButton*       zoomToAllButtonX;
 ToggleButton*       zoomToAllButtonX;
 ScrollBar*          scrollBarX;
 SymbolButton*       zoomInButtonY;
 SymbolButton*       zoomOutButtonY;
 ToggleButton*       zoomToAllButtonY;
 ScrollBar*          scrollBarY;
 ToggleButton*       zoomToAllButtonXY;


 int                 oldMouseDistanceX, oldMouseDistanceY;

 double zoomFactorPerClickX, zoomFactorPerClickY;
 /**< The factor by which is zoomed in/out on one click on a plus/minus 
      button for x- and y-axis seperately. */

 double shiftPerClickX, shiftPerClickY;
 /**< The amount by which the CoordinateSystem3D is shifted left/right or 
      up/down on one click on an arrow-button on the scrollbar. */

 int verticalMouseWheelMode;
  /**< The mode of the(vertical) mousewheel - can scroll or zoom horizontally, 
       vertically, both or none. */

 // a flag which allows to suppress all the zooming and scrolling widgets, 
 // leaving more space for the actual coordinate system:
 bool supressZoomability;

 // thickness of the scrollbars and edge-length of the buttons:
 int zoomerSize;

 virtual void updateScrollbars();
};

#endif  // end of #ifndef __JUCE_COORDINATESYSTEM3DZOOMER_JUCEHEADER__