/**

This class represents the visible range for a CoordinateSystem (and is used 
inside this class).

*/

#ifndef __JUCE_COORDINATESYSTEMRANGE_JUCEHEADER__
#define __JUCE_COORDINATESYSTEMRANGE_JUCEHEADER__

#include <juce.h>

class CoordinateSystemRange
{

public:


  CoordinateSystemRange(double initMinX = -2.2, double initMaxX = +2.2,
    double initMinY = -2.2, double initMaxY = +2.2);
  /**< Standard Constructor. */

  virtual ~CoordinateSystemRange();  
  /**< Destructor. */

  virtual double getMinX() const { return minX; }
  /**< Returns the minimum x-value. */

  virtual double getMaxX() const { return maxX; }
  /**< Returns the maximum x-value. */

  virtual double getMinY() const { return minY; }
  /**< Returns the minimum y-value. */

  virtual double getMaxY() const { return maxY; }
  /**< Returns the maximum y-value. */

  virtual void setMinX(double newMinX);
  /**< Sets the minimum x-value. */

  virtual void setMaxX(double newMaxX);
  /**< Sets the maximum x-value. */

  virtual void setRangeX(double newMinX, double newMaxX);
  /**< Sets up the minimum and maximum value for x .*/

  virtual void setMinY(double newMinY);
  /**< Sets the minimum y-value. */

  virtual void setMaxY(double newMaxY); 
  /**< Sets the maximum y-value. */

  virtual void setRangeY(double newMinY, double newMaxY);
  /**< Sets up the minimum and maximum value for y .*/

  virtual void clipRange(CoordinateSystemRange rangeToClipTo);
  /**< Lets the range represented by the member-variables be clipped to another range such that
  the minima/maxima are no smaller/larger than those of the rangeToClipTo. */

protected:

  double minX, maxX, minY, maxY;
};

#endif  // __JUCE_COORDINATESYSTEMRANGE_JUCEHEADER__
