/**

This class manages presets stored as xml-files 

*/

#ifndef __JUCE_FILEMANAGER_JUCEHEADER__
#define __JUCE_FILEMANAGER_JUCEHEADER__

#include "Comparators.h"

class FileManager

{

public:

  FileManager();  
  ///< Constructor.

  virtual ~FileManager();  
  ///< Destructor.

protected:

  virtual void openLoadingDialog(const String &dialogTitle = String(T("Load File")) );
  /**< Opens a dialog-box to load from a file. */

  virtual void openSavingDialog(const String &dialogTitle = String(T("Save File")) );
  /**< Opens a dialog-box to save to a file. */

  virtual void updateFileList();
  /**< Updates the array which contains all the relevant files in order to skip through them
       via plus/minus buttons. */

  virtual void incrementCurrentFile();
  /**< Adjusts the currentFile... variables such that they point to the next file in the current 
       directory. The actual loading actions must be implemented by subclasses. */

  virtual void decrementCurrentFile();
  /**< Adjusts the currentFile... variables such that they point to the previous file in the 
       current directory. The actual loading actions must be implemented by subclasses. */

  virtual File getCurrentFile();
  /**< Returns the currently marked file (in order to load it or whatever else. */

  virtual void retrieveCurrentDirectoryFromPreferences(String xmlAttributeToLookFor);
  /**< Tries to set the currentDirectory to a default value which is expected to be specified by
       some XML-attribute in a Preferences.xml file which is assumed to be in the application 
       directory. */

  // for remebering the directories in which was most recntly browsed for 
  // presets:
  bool   fileIsUnsaved;
  String currentFileName;
  String currentFileNameWithStar;
  String currentFileFullPath;
  String currentFileRelativePath;
  String currentDirectory;
  String defaultExtension;

  String             wildcardPatterns;
  WildcardFileFilter fileFilter;

  OwnedArray<File> filesInCurrentDirectory; // holds all relevant files in the current directory
  int currentFileIndex;                     // index of the current preset in the array
  int numFilesInCurrentDirectory;
  FileComparator fileComparator; // this object is needed to sort the preset-array
};

#endif  // end of #ifndef __JUCE_FILEMANAGER_JUCEHEADER__