#ifndef __JUCE_GRAPHICSTOOLS_JUCEHEADER__
#define __JUCE_GRAPHICSTOOLS_JUCEHEADER__

#include <juce.h>

void fillRectWithBilinearGradient(
 Graphics &g, 
 int x, int y, int w, int h, 
 Colour topLeftColour     = Colours::red, 
 Colour topRightColour    = Colours::green, 
 Colour bottomLeftColour  = Colours::blue, 
 Colour bottomRightColour = Colours::white);
/**< Fills an rectangle with a 4-point bilinear gradient. */

void fillRectWithBilinearGradient(
 Graphics &g, Rectangle r, 
 Colour topLeftColour     = Colours::red, 
 Colour topRightColour    = Colours::green, 
 Colour bottomLeftColour  = Colours::blue, 
 Colour bottomRightColour = Colours::white);
/**< Fills an rectangle with a 4-point bilinear gradient. */

void fillRectWithBilinearGrayScaleGradient(
 Graphics &g, 
 int x, int y, int w, int h, 
 float topLeftWhite     = 0.9, 
 float topRightWhite    = 0.7, 
 float bottomLeftWhite  = 0.7, 
 float bottomRightWhite = 0.9);
/**< Fills an rectangle with a 4-point bilinear gradient. */

void fillRectWithDefaultBackground(Graphics &g, int x, int y, int w, int h);
/**< Fills an rectangle with the default background. */

void fillRectWithDefaultBackground(Graphics &g, Rectangle r);
/**< Fills an rectangle with the default background. */

void drawTriangle(Graphics &g, float x1, float y1, float x2, float y2, 
                  float x3, float y3, bool fill);
/**< Draws a triangle based on 3 points and optionally fills it. */

void fitLineToRectangle(double &x1, double &y1, double &x2, double &y2, 
                        double xMin, double yMin, double xMax, double yMax);
/**< Prolongs and/or shortens the line through x1,y1 and x2,y2 such that it fits into the rectangle
bounded by xMin, yMin, and xMax yMax. */

void clipLineToRectangle(double &x1, double &y1, double &x2, double &y2, 
                         double xMin, double yMin, double xMax, double yMax);
/**< Clips a line off at boundaries xMin, yMin, and xMax yMax. */


#endif