#ifndef __JUCE_IMAGESAVINGDIALOG_JUCEHEADER__
#define __JUCE_IMAGESAVINGDIALOG_JUCEHEADER__

#include "CoordinateSystem.h"

/**

This class is to be used as content component for a DialogWindow object for saving images - 
it is mainly intended to export the contents of a Component to image files.

*/

class ImageSavingDialog 
  : public Component, public ButtonListener, public ComboBoxListener, public LabelListener
{

public:

  /** Constructor. */
  ImageSavingDialog(CoordinateSystem *owner = NULL, int defaultWidth = 320, 
    int defaultHeight = 320, const String &defaultFormat = String(T("png")),  
    const File& defaultTargetFile = File::nonexistent);

  virtual ~ImageSavingDialog();
  /**< Destructor. */

  virtual int getSelectedPixelWidth();
  /**< Returns the desired pixel-width which as chosen by the user in the dialog. */

  virtual int getSelectedPixelHeight();
  /**< Returns the desired pixel-height which as chosen by the user in the dialog. */

  virtual void buttonClicked(Button *buttonThatWasClicked);
  /**< Implements the purely virtual buttonClicked()-method of the ButtonListener base-class. */

  virtual void comboBoxChanged(ComboBox *comboBoxThatHasChanged);
  /**< Implements the purely virtual comboBoxChanged()-method of the ComboBoxListener base-class.*/

  virtual void labelTextChanged(Label *labelThatHasChanged); 
  /**< Implements the purely virtual labelTextChanged()-method of the LablelListener base-class. */
  
protected:

  void saveNow();

  CoordinateSystem *ownerSystem;

  Label        *targetFileLabel;
  Label        *targetFileEditLabel;
  TextButton   *browseButton;
  Label        *pixelWidthLabel;
  Label        *pixelWidthEditLabel;
  Label        *pixelHeightLabel;
  Label        *pixelHeightEditLabel;
  Label        *formatLabel;
  ComboBox     *formatComboBox;
  TextButton   *saveButton;

};

#endif	// __JUCE_IMAGESAVINGDIALOG_JUCEHEADER__
