#include "StateFileManager.h"

//----------------------------------------------------------------------------
// construction/destruction:

StateFileManager::StateFileManager() 
{
  stateNameString  = String(T("State"));
  wildcardPatterns = String(T("*.xml"));
  defaultExtension = String(T(".xml"));
}

StateFileManager::~StateFileManager()
{

}


//----------------------------------------------------------------------------
// state-/preset-management:

XmlElement* StateFileManager::getStateAsXml() const
{
  // return an empty XmlElement:
  return new XmlElement(stateNameString);
}

bool StateFileManager::setStateFromXml(const XmlElement &xmlState)
{
  // return false by default - subclasses should override this function, do 
  // something meaningful with the passed XmlElement, and then possibly return
  // true:
  return false;
}

void StateFileManager::loadStateFromXml(const File& fileToLoadFrom)
{
  XmlDocument myDocument(fileToLoadFrom);
  XmlElement* xmlState = myDocument.getDocumentElement();

  if( xmlState != NULL )
  {
    setStateFromXml(*xmlState);
    delete xmlState;
  }
}

void StateFileManager::saveStateToXml(const File& fileToSaveTo)
{
  // obtain the current state as XmlElement::
  XmlElement* xmlState = getStateAsXml();

  // now we can turn the XmlElement thing into a text document:
  String myXmlDoc = xmlState->createDocument(String::empty);

  if( fileToSaveTo.existsAsFile() )
    fileToSaveTo.deleteFile();

  fileToSaveTo.create();
  fileToSaveTo.appendText(myXmlDoc);
}

void StateFileManager::openLoadingDialog(const String &dialogTitle)
{
  FileManager::openLoadingDialog();

  // the inherited member currentFileFullPath has now be assigned to the file which should be 
  // loaded - so create a file from the String and do it:
  loadStateFromXml(File(currentFileFullPath));
}

void StateFileManager::openSavingDialog(const String &dialogTitle)
{
  FileManager::openSavingDialog();

  // the inherited member currentFileFullPath has now be assigned to the file which should be 
  // saved - so create a file from the String and do it:
  saveStateToXml(File(currentFileFullPath));
}





