/**

This class manages presets stored as xml-files 

*/

#ifndef __JUCE_STATEFILEMANAGER_JUCEHEADER__
#define __JUCE_STATEFILEMANAGER_JUCEHEADER__

#include "FileManager.h"

class StateFileManager : public FileManager
{

public:

  StateFileManager();  
  ///< Constructor.

  virtual ~StateFileManager();  
  ///< Destructor.

  virtual XmlElement* getStateAsXml() const;
  /**< Must be overriden by subclasses and is intended to return the state
  of the StateFileManager (i.e. the settings of all relevant parameters) in form
  of an XmlElement. */

  virtual bool setStateFromXml(const XmlElement& xmlState);
  /**< Must be overriden by subclasses and is intended to recall the state
  of the StateFileManager (i.e. the settings of all relevant parameters) from
  an XmlElement. The boolean return value informs, if all required 
  attributes were found in the passed XmlElement. */

protected:

  // open a dialog-box to load/save from/to a file:
  virtual void openLoadingDialog(const String &dialogTitle = String(T("Load File")) );
  virtual void openSavingDialog(const String &dialogTitle = String(T("Save File")) );

  // the actual load/save functions:
  virtual void loadStateFromXml(const File& fileToLoadFrom);
  virtual void saveStateToXml(const File& fileToSaveTo);

  String stateNameString;
};

#endif  // end of #ifndef __JUCE_STATEFILEMANAGER_JUCEHEADER__