#include "SurfacePlot.h"

SurfacePlot::SurfacePlot(const String& name) : CoordinateSystem3D(name)
{
  numValuesX1           = 0;
  numValuesX2           = 0;
  valuesX1              = NULL;
  valuesX2              = NULL;
  valuesY               = NULL;
  surfaceColour	        = Colours::black.withAlpha(0.25f);
  meshGridColour	       = Colours::black.withAlpha(0.25f);
  dotColour	            = Colours::black;
  drawAsCloud           = false;
  plotSurfaceWithMesh   = true;
  plotSurfaceWithDots   = false;
  gridMode              = TRIANGULAR;

  // TEST: 
  numValuesX1            = 9;
  valuesX1               = new double[numValuesX1];
  numValuesX2            = 9;
  valuesX2               = new double[numValuesX2];
  double* valuesYAsArray = new double[numValuesX1*numValuesX2];

  // generate the test-data:
  int x1, x2;
  for(x1=0; x1<numValuesX1; x1++)
  {
    valuesX1[x1] = 1.0*(x1-4);
    for(x2=0; x2<numValuesX2; x2++)
    {
      valuesX2[x2] = 1.0*(x2-4);
      //valuesYAsArray[x1*numValuesX2+x2] = sqrt(fabs(valuesX1[x1]+valuesX2[x2]));
      valuesYAsArray[x1*numValuesX2+x2] 
      = 1.0 / (1.0 + sqrt(fabs(valuesX1[x1]*valuesX2[x2])));
    }
  }

  valuesY = new double*[numValuesX1];
  for(x1=0; x1<numValuesX1; x1++)
    valuesY[x1] = &(valuesYAsArray[x1*numValuesX2]);

  //setRotationAboutAxis(0.0, Y_AXIS);
  //setRotationAboutAxis(90.0, X_AXIS);

  int dummy = 1;

}

SurfacePlot::~SurfacePlot(void)
{
  deleteAllChildren();
}

//----------------------------------------------------------------------------

void SurfacePlot::setSurfaceValues(int newNumValuesX1, double* newValuesX1,
                                   int newNumValuesX2, double* newValuesX2,
                                   double** newValuesY)
{
  numValuesX1 = newNumValuesX1;
  valuesX1    = newValuesX1;
  numValuesX2 = newNumValuesX2;
  valuesX2    = newValuesX2;
  valuesY     = newValuesY;
}

void SurfacePlot::invalidatePointers()
{
  numValuesX1 = 0;
  valuesX1    = NULL;
  numValuesX2 = 0;
  valuesX2    = NULL;
  valuesY     = NULL;
}

//----------------------------------------------------------------------------

void SurfacePlot::paint(Graphics &g)
{
  // call the paint-method of the CoordinateSystem3D base class, which draws 
  // the axes, labels, grids, etc.:
  CoordinateSystem3D::paint(g);

  // plot the surface or cloud:
  if( drawAsCloud )
    plotCloud(g);
  else  
    plotSurface(g, plotSurfaceWithMesh, plotSurfaceWithDots);
}

//----------------------------------------------------------------------------

void SurfacePlot::plotSurface(Graphics &g, bool plotWithMeshGrid, 
                              bool plotWithDots)
{
  double x1, x2, x3, x4, y1, y2, y3, y4, z1, z2, z3, z4;

  Path surfaceSegment;

  if( gridMode == QUADRANGULAR )
  {

    for(int index1=0; index1<numValuesX1-1; index1++)
    {
      for(int index2=0; index2<numValuesX2-1; index2++)
      {
        x1 = valuesX1[index1];
        z1 = valuesX2[index2];
        y1 = valuesY[index1][index2];
        transformToComponentsCoordinates(x1, y1, z1);

        x2 = valuesX1[index1+1];
        z2 = valuesX2[index2];
        y2 = valuesY[index1+1][index2];
        transformToComponentsCoordinates(x2, y2, z2);

        x3 = valuesX1[index1+1];
        z3 = valuesX2[index2+1];
        y3 = valuesY[index1+1][index2+1];
        transformToComponentsCoordinates(x3, y3, z3);

        x4 = valuesX1[index1];
        z4 = valuesX2[index2+1];
        y4 = valuesY[index1][index2+1];
        transformToComponentsCoordinates(x4, y4, z4);

        surfaceSegment.clear();

        surfaceSegment.addQuadrilateral((float) x1, (float) y1, (float) x2, 
          (float) y2, (float) x3, (float) y3, (float) x4, (float) y4);

        g.setColour(surfaceColour);
        g.fillPath(surfaceSegment);

        if( plotWithMeshGrid == true )
        {
          g.setColour(meshGridColour);
          g.strokePath(surfaceSegment, PathStrokeType::curved);
        }

      }
    } 
  }//end of  if( gridMode == QUADRANGULAR )
  else if ( gridMode == TRIANGULAR )
  {

    for(int index1=0; index1<numValuesX1-1; index1++)
    {
      for(int index2=0; index2<numValuesX2-1; index2++)
      {
        x1 = valuesX1[index1];
        z1 = valuesX2[index2];
        y1 = valuesY[index1][index2];
        transformToComponentsCoordinates(x1, y1, z1);

        x2 = valuesX1[index1+1];
        z2 = valuesX2[index2];
        y2 = valuesY[index1+1][index2];
        transformToComponentsCoordinates(x2, y2, z2);

        x3 = valuesX1[index1+1];
        z3 = valuesX2[index2+1];
        y3 = valuesY[index1+1][index2+1];
        transformToComponentsCoordinates(x3, y3, z3);

        x4 = valuesX1[index1];
        z4 = valuesX2[index2+1];
        y4 = valuesY[index1][index2+1];
        transformToComponentsCoordinates(x4, y4, z4);

        surfaceSegment.clear();
        surfaceSegment.addTriangle((float) x1, (float) y1, (float) x2, 
          (float) y2, (float) x4, (float) y4);
        g.setColour(surfaceColour);
        g.fillPath(surfaceSegment);
        if( plotWithMeshGrid == true )
        {
          g.setColour(meshGridColour);
          g.strokePath(surfaceSegment, PathStrokeType::curved);
        }

        surfaceSegment.clear();
        surfaceSegment.addTriangle((float) x2, (float) y2, (float) x3, 
          (float) y3, (float) x4, (float) y4);
        g.setColour(surfaceColour);
        g.fillPath(surfaceSegment);
        if( plotWithMeshGrid == true )
        {
          g.setColour(meshGridColour);
          g.strokePath(surfaceSegment, PathStrokeType::curved);
        }

      }

    } 

  }


  if( plotWithDots == true )
    plotCloud(g);
}

//----------------------------------------------------------------------------

void SurfacePlot::plotCloud(Graphics &g)
{
  double x, y, z;
  g.setColour(dotColour);

  for(int index1=0; index1<numValuesX1; index1++)
  {
    for(int index2=0; index2<numValuesX2; index2++)
    {
      x = valuesX1[index1];
      z = valuesX2[index2];
      y = valuesY[index1][index2];

      transformToComponentsCoordinates(x, y, z);

      g.fillEllipse((float) (x-2.0),(float) (y-2.0), 4.f, 4.f);
    }
  }
}

