#include "rosic_ExpressionEvaluator.h"
#include "rosic_ExpressionEvaluatorFunctions.h"
using namespace rosic;

//-------------------------------------------------------------------------------------------------
// construction/destruction:

ExpressionEvaluator::ExpressionEvaluator()
{
  expressionString = NULL;

  valueList.AddDefaultValues();
  valueList.Add("pi", PI,    false);
  valueList.Add("e",  EULER, false);
  //...

  functionList.AddDefaultFunctions();
  functionList.Add(new FunctionFactoryCheby() );
  functionList.Add(new FunctionFactoryGauss() );
  functionList.Add(new FunctionFactoryQuant() );
  functionList.Add(new FunctionFactoryQuantToBits() );
  functionList.Add(new FunctionFactoryPow() );
  functionList.Add(new FunctionFactoryPowBipolar() );
  functionList.Add(new FunctionFactorySaw() );
  functionList.Add(new FunctionFactorySign() );
  functionList.Add(new FunctionFactorySoftClip() );
  //...

  expression.SetValueList(&valueList);
  expression.SetFunctionList(&functionList);

}

ExpressionEvaluator::~ExpressionEvaluator()
{
  if( expressionString != NULL )
    delete[] expressionString;
}

//-------------------------------------------------------------------------------------------------
// parse/evaluate expressions:

bool ExpressionEvaluator::setExpressionString(char* newExpressionString)
{
  // copy the passed c-string into our member variable:
  if( expressionString != NULL )
  {
    delete[] expressionString;
    expressionString = NULL;
  }
  int length = strlen(newExpressionString);
  expressionString = new char[length+1];
  if( expressionString != NULL )
  {
    for(int c=0; c<=length; c++)
      expressionString[c] = newExpressionString[c];

    // try to parse the expression and return whether this was succesful or not:
    return parseExpression();
  }
  else
    return false;
}

bool ExpressionEvaluator::parseExpression()
{
  try
  { 
    expression.Parse(expressionString);
  }
  catch( ExprEval::Exception &theException )
  {
    theException.GetValue(); // dummy instruction to suppress 'unreferenced local variable' warning
    return false;
  }

  // no exception was caught, so the parsing was successful:
  return true;
}

double ExpressionEvaluator::evaluateExpression()
{
  return expression.Evaluate();
}

//-------------------------------------------------------------------------------------------------
// variable assignments:

void ExpressionEvaluator::assignVariable(char* name, double value)
{
  // check, if a variable with this name already exists, if yes, overwrite its value, if not add it 
  // and assign the value
  double* address = valueList.GetAddress(name);
  if( address != NULL )
  {
    *address = value;
  }
  else
  {
    valueList.Add(name, value, false);
    address = valueList.GetAddress(name);  
    *address = value;
  }
}

double* ExpressionEvaluator::getVariableAddress(char* name)
{
  return valueList.GetAddress(name);
}

//-------------------------------------------------------------------------------------------------
// inquiry:

char* ExpressionEvaluator::getExpressionString()
{
  return expressionString;
}
